#pragma once
#include "stdexcept"
#include "stddef.h"
#include <array>
#include <cstdint>
#include <new>
#include <string>
#include <type_traits>
#include <utility>
#if defined(_WIN32)
#include <basetsd.h>
#endif

namespace rust {
inline namespace cxxbridge05 {
// #include "rust/cxx.h"

#ifndef CXXBRIDGE05_PANIC
#define CXXBRIDGE05_PANIC
template <typename Exception>
void panic [[noreturn]] (const char *msg);
#endif // CXXBRIDGE05_PANIC

struct unsafe_bitcopy_t;

#ifndef CXXBRIDGE05_RUST_STRING
#define CXXBRIDGE05_RUST_STRING
class String final {
public:
  String() noexcept;
  String(const String &) noexcept;
  String(String &&) noexcept;
  ~String() noexcept;

  String(const std::string &);
  String(const char *);
  String(const char *, size_t);

  String &operator=(const String &) noexcept;
  String &operator=(String &&) noexcept;

  explicit operator std::string() const;

  const char *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  String(unsafe_bitcopy_t, const String &) noexcept;

private:
  std::array<uintptr_t, 3> repr;
};
#endif // CXXBRIDGE05_RUST_STRING

#ifndef CXXBRIDGE05_RUST_BOX
#define CXXBRIDGE05_RUST_BOX
template <typename T>
class Box final {
public:
  using value_type = T;
  using const_pointer =
      typename std::add_pointer<typename std::add_const<T>::type>::type;
  using pointer = typename std::add_pointer<T>::type;

  Box(const Box &);
  Box(Box &&) noexcept;
  ~Box() noexcept;

  explicit Box(const T &);
  explicit Box(T &&);

  Box &operator=(const Box &);
  Box &operator=(Box &&) noexcept;

  const T *operator->() const noexcept;
  const T &operator*() const noexcept;
  T *operator->() noexcept;
  T &operator*() noexcept;

  template <typename... Fields>
  static Box in_place(Fields &&...);

  static Box from_raw(T *) noexcept;

  T *into_raw() noexcept;

private:
  Box() noexcept;
  void uninit() noexcept;
  void drop() noexcept;
  T *ptr;
};

template <typename T>
Box<T>::Box(const Box &other) : Box(*other) {}

template <typename T>
Box<T>::Box(Box &&other) noexcept : ptr(other.ptr) {
  other.ptr = nullptr;
}

template <typename T>
Box<T>::Box(const T &val) {
  this->uninit();
  ::new (this->ptr) T(val);
}

template <typename T>
Box<T>::Box(T &&val) {
  this->uninit();
  ::new (this->ptr) T(std::move(val));
}

template <typename T>
Box<T>::~Box() noexcept {
  if (this->ptr) {
    this->drop();
  }
}

template <typename T>
Box<T> &Box<T>::operator=(const Box &other) {
  if (this != &other) {
    if (this->ptr) {
      **this = *other;
    } else {
      this->uninit();
      ::new (this->ptr) T(*other);
    }
  }
  return *this;
}

template <typename T>
Box<T> &Box<T>::operator=(Box &&other) noexcept {
  if (this->ptr) {
    this->drop();
  }
  this->ptr = other.ptr;
  other.ptr = nullptr;
  return *this;
}

template <typename T>
const T *Box<T>::operator->() const noexcept {
  return this->ptr;
}

template <typename T>
const T &Box<T>::operator*() const noexcept {
  return *this->ptr;
}

template <typename T>
T *Box<T>::operator->() noexcept {
  return this->ptr;
}

template <typename T>
T &Box<T>::operator*() noexcept {
  return *this->ptr;
}

template <typename T>
template <typename... Fields>
Box<T> Box<T>::in_place(Fields &&... fields) {
  Box box;
  box.uninit();
  ::new (box.ptr) T{std::forward<Fields>(fields)...};
  return box;
}

template <typename T>
Box<T> Box<T>::from_raw(T *raw) noexcept {
  Box box;
  box.ptr = raw;
  return box;
}

template <typename T>
T *Box<T>::into_raw() noexcept {
  T *raw = this->ptr;
  this->ptr = nullptr;
  return raw;
}

template <typename T>
Box<T>::Box() noexcept {}
#endif // CXXBRIDGE05_RUST_BOX

#ifndef CXXBRIDGE05_RUST_BITCOPY
#define CXXBRIDGE05_RUST_BITCOPY
struct unsafe_bitcopy_t final {
  explicit unsafe_bitcopy_t() = default;
};

constexpr unsafe_bitcopy_t unsafe_bitcopy{};
#endif // CXXBRIDGE05_RUST_BITCOPY

#ifndef CXXBRIDGE05_RUST_VEC
#define CXXBRIDGE05_RUST_VEC
template <typename T>
class Vec final {
public:
  using value_type = T;

  Vec() noexcept;
  Vec(Vec &&) noexcept;
  ~Vec() noexcept;

  Vec &operator=(Vec &&) noexcept;

  size_t size() const noexcept;
  bool empty() const noexcept;
  const T *data() const noexcept;
  T *data() noexcept;

  const T &operator[](size_t n) const noexcept;
  const T &at(size_t n) const;

  const T &front() const;
  const T &back() const;

  void reserve(size_t new_cap);
  void push_back(const T &value);
  void push_back(T &&value);
  template <typename... Args>
  void emplace_back(Args &&... args);

  class const_iterator final {
  public:
    using difference_type = ptrdiff_t;
    using value_type = typename std::add_const<T>::type;
    using pointer =
        typename std::add_pointer<typename std::add_const<T>::type>::type;
    using reference = typename std::add_lvalue_reference<
        typename std::add_const<T>::type>::type;
    using iterator_category = std::forward_iterator_tag;

    const T &operator*() const noexcept;
    const T *operator->() const noexcept;
    const_iterator &operator++() noexcept;
    const_iterator operator++(int) noexcept;
    bool operator==(const const_iterator &) const noexcept;
    bool operator!=(const const_iterator &) const noexcept;

  private:
    friend class Vec;
    const void *pos;
    size_t stride;
  };

  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;

  Vec(unsafe_bitcopy_t, const Vec &) noexcept;

private:
  static size_t stride() noexcept;
  void reserve_total(size_t cap) noexcept;
  void set_len(size_t len) noexcept;
  void drop() noexcept;

  std::array<uintptr_t, 3> repr;
};

template <typename T>
Vec<T>::Vec(Vec &&other) noexcept {
  this->repr = other.repr;
  new (&other) Vec();
}

template <typename T>
Vec<T>::~Vec() noexcept {
  this->drop();
}

template <typename T>
Vec<T> &Vec<T>::operator=(Vec &&other) noexcept {
  if (this != &other) {
    this->drop();
    this->repr = other.repr;
    new (&other) Vec();
  }
  return *this;
}

template <typename T>
bool Vec<T>::empty() const noexcept {
  return size() == 0;
}

template <typename T>
T *Vec<T>::data() noexcept {
  return const_cast<T *>(const_cast<const Vec<T> *>(this)->data());
}

template <typename T>
const T &Vec<T>::operator[](size_t n) const noexcept {
  auto data = reinterpret_cast<const char *>(this->data());
  return *reinterpret_cast<const T *>(data + n * this->stride());
}

template <typename T>
const T &Vec<T>::at(size_t n) const {
  if (n >= this->size()) {
    panic<std::out_of_range>("rust::Vec index out of range");
  }
  return (*this)[n];
}

template <typename T>
const T &Vec<T>::front() const {
  return (*this)[0];
}

template <typename T>
const T &Vec<T>::back() const {
  return (*this)[this->size() - 1];
}

template <typename T>
void Vec<T>::reserve(size_t new_cap) {
  this->reserve_total(new_cap);
}

template <typename T>
void Vec<T>::push_back(const T &value) {
  this->emplace_back(value);
}

template <typename T>
void Vec<T>::push_back(T &&value) {
  this->emplace_back(std::move(value));
}

template <typename T>
template <typename... Args>
void Vec<T>::emplace_back(Args &&... args) {
  auto size = this->size();
  this->reserve_total(size + 1);
  ::new (reinterpret_cast<T *>(reinterpret_cast<char *>(this->data()) +
                               size * this->stride()))
      T(std::forward<Args>(args)...);
  this->set_len(size + 1);
}

template <typename T>
const T &Vec<T>::const_iterator::operator*() const noexcept {
  return *static_cast<const T *>(this->pos);
}

template <typename T>
const T *Vec<T>::const_iterator::operator->() const noexcept {
  return static_cast<const T *>(this->pos);
}

template <typename T>
typename Vec<T>::const_iterator &Vec<T>::const_iterator::operator++() noexcept {
  this->pos = static_cast<const uint8_t *>(this->pos) + this->stride;
  return *this;
}

template <typename T>
typename Vec<T>::const_iterator
Vec<T>::const_iterator::operator++(int) noexcept {
  auto ret = const_iterator(*this);
  this->pos = static_cast<const uint8_t *>(this->pos) + this->stride;
  return ret;
}

template <typename T>
bool Vec<T>::const_iterator::operator==(
    const const_iterator &other) const noexcept {
  return this->pos == other.pos;
}

template <typename T>
bool Vec<T>::const_iterator::operator!=(
    const const_iterator &other) const noexcept {
  return this->pos != other.pos;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::begin() const noexcept {
  const_iterator it;
  it.pos = this->data();
  it.stride = this->stride();
  return it;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::end() const noexcept {
  const_iterator it = this->begin();
  it.pos = static_cast<const uint8_t *>(it.pos) + it.stride * this->size();
  return it;
}

template <typename T>
Vec<T>::Vec(unsafe_bitcopy_t, const Vec &bits) noexcept : repr(bits.repr) {}
#endif // CXXBRIDGE05_RUST_VEC

#ifndef CXXBRIDGE05_RUST_ISIZE
#define CXXBRIDGE05_RUST_ISIZE
#if defined(_WIN32)
using isize = SSIZE_T;
#else
using isize = ssize_t;
#endif
#endif // CXXBRIDGE05_RUST_ISIZE
} // namespace cxxbridge05
} // namespace rust

namespace newsboat {
  namespace cliargsparser {
    namespace bridged {
      struct CliArgsParser;
    }
  }
}

namespace newsboat {
namespace cliargsparser {
namespace bridged {
::rust::Box<::newsboat::cliargsparser::bridged::CliArgsParser> create(::rust::Vec<::rust::String> argv) noexcept;

bool do_import(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool do_export(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool do_vacuum(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool do_cleanup(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

uint64_t do_show_version(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool silent(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool using_nonstandard_configs(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool should_print_usage(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool refresh_on_start(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

::rust::String importfile(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

::rust::String program_name(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

::rust::String display_msg(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool return_code(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::isize &value) noexcept;

bool readinfo_import_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

bool readinfo_export_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

bool url_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

bool lock_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

bool cache_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

bool config_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

bool log_file(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, ::rust::String &path) noexcept;

::rust::Vec<::rust::String> cmds_to_execute(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser) noexcept;

bool log_level(const ::newsboat::cliargsparser::bridged::CliArgsParser &cliargsparser, int8_t &level) noexcept;
} // namespace bridged
} // namespace cliargsparser
} // namespace newsboat
