#pragma once
#include "stdexcept"
#include "stddef.h"
#include <array>
#include <cstddef>
#include <cstdint>
#include <new>
#include <string>
#include <type_traits>
#include <utility>
#if defined(_WIN32)
#include <basetsd.h>
#endif

namespace rust {
inline namespace cxxbridge05 {
// #include "rust/cxx.h"

#ifndef CXXBRIDGE05_PANIC
#define CXXBRIDGE05_PANIC
template <typename Exception>
void panic [[noreturn]] (const char *msg);
#endif // CXXBRIDGE05_PANIC

struct unsafe_bitcopy_t;

namespace {
template <typename T>
class impl;
} // namespace

#ifndef CXXBRIDGE05_RUST_STRING
#define CXXBRIDGE05_RUST_STRING
class String final {
public:
  String() noexcept;
  String(const String &) noexcept;
  String(String &&) noexcept;
  ~String() noexcept;

  String(const std::string &);
  String(const char *);
  String(const char *, size_t);

  String &operator=(const String &) noexcept;
  String &operator=(String &&) noexcept;

  explicit operator std::string() const;

  const char *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  String(unsafe_bitcopy_t, const String &) noexcept;

private:
  std::array<uintptr_t, 3> repr;
};
#endif // CXXBRIDGE05_RUST_STRING

#ifndef CXXBRIDGE05_RUST_STR
#define CXXBRIDGE05_RUST_STR
class Str final {
public:
  Str() noexcept;
  Str(const std::string &);
  Str(const char *);
  Str(const char *, size_t);
  Str(std::string &&) = delete;

  Str &operator=(const Str &) noexcept = default;

  explicit operator std::string() const;

  const char *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  Str(const Str &) noexcept = default;
  ~Str() noexcept = default;

private:
  friend impl<Str>;
  const char *ptr;
  size_t len;
};

inline const char *Str::data() const noexcept { return this->ptr; }

inline size_t Str::size() const noexcept { return this->len; }

inline size_t Str::length() const noexcept { return this->len; }
#endif // CXXBRIDGE05_RUST_STR

#ifndef CXXBRIDGE05_RUST_SLICE
#define CXXBRIDGE05_RUST_SLICE
template <typename T>
class Slice final {
public:
  Slice() noexcept;
  Slice(const T *, size_t count) noexcept;

  Slice &operator=(const Slice<T> &) noexcept = default;

  const T *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  Slice(const Slice<T> &) noexcept = default;
  ~Slice() noexcept = default;

private:
  friend impl<Slice>;
  const T *ptr;
  size_t len;
};

template <typename T>
Slice<T>::Slice() noexcept : ptr(reinterpret_cast<const T *>(this)), len(0) {}

template <typename T>
Slice<T>::Slice(const T *s, size_t count) noexcept : ptr(s), len(count) {}

template <typename T>
const T *Slice<T>::data() const noexcept {
  return this->ptr;
}

template <typename T>
size_t Slice<T>::size() const noexcept {
  return this->len;
}

template <typename T>
size_t Slice<T>::length() const noexcept {
  return this->len;
}
#endif // CXXBRIDGE05_RUST_SLICE

#ifndef CXXBRIDGE05_RUST_BITCOPY
#define CXXBRIDGE05_RUST_BITCOPY
struct unsafe_bitcopy_t final {
  explicit unsafe_bitcopy_t() = default;
};

constexpr unsafe_bitcopy_t unsafe_bitcopy{};
#endif // CXXBRIDGE05_RUST_BITCOPY

#ifndef CXXBRIDGE05_RUST_VEC
#define CXXBRIDGE05_RUST_VEC
template <typename T>
class Vec final {
public:
  using value_type = T;

  Vec() noexcept;
  Vec(Vec &&) noexcept;
  ~Vec() noexcept;

  Vec &operator=(Vec &&) noexcept;

  size_t size() const noexcept;
  bool empty() const noexcept;
  const T *data() const noexcept;
  T *data() noexcept;

  const T &operator[](size_t n) const noexcept;
  const T &at(size_t n) const;

  const T &front() const;
  const T &back() const;

  void reserve(size_t new_cap);
  void push_back(const T &value);
  void push_back(T &&value);
  template <typename... Args>
  void emplace_back(Args &&... args);

  class const_iterator final {
  public:
    using difference_type = ptrdiff_t;
    using value_type = typename std::add_const<T>::type;
    using pointer =
        typename std::add_pointer<typename std::add_const<T>::type>::type;
    using reference = typename std::add_lvalue_reference<
        typename std::add_const<T>::type>::type;
    using iterator_category = std::forward_iterator_tag;

    const T &operator*() const noexcept;
    const T *operator->() const noexcept;
    const_iterator &operator++() noexcept;
    const_iterator operator++(int) noexcept;
    bool operator==(const const_iterator &) const noexcept;
    bool operator!=(const const_iterator &) const noexcept;

  private:
    friend class Vec;
    const void *pos;
    size_t stride;
  };

  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;

  Vec(unsafe_bitcopy_t, const Vec &) noexcept;

private:
  static size_t stride() noexcept;
  void reserve_total(size_t cap) noexcept;
  void set_len(size_t len) noexcept;
  void drop() noexcept;

  std::array<uintptr_t, 3> repr;
};

template <typename T>
Vec<T>::Vec(Vec &&other) noexcept {
  this->repr = other.repr;
  new (&other) Vec();
}

template <typename T>
Vec<T>::~Vec() noexcept {
  this->drop();
}

template <typename T>
Vec<T> &Vec<T>::operator=(Vec &&other) noexcept {
  if (this != &other) {
    this->drop();
    this->repr = other.repr;
    new (&other) Vec();
  }
  return *this;
}

template <typename T>
bool Vec<T>::empty() const noexcept {
  return size() == 0;
}

template <typename T>
T *Vec<T>::data() noexcept {
  return const_cast<T *>(const_cast<const Vec<T> *>(this)->data());
}

template <typename T>
const T &Vec<T>::operator[](size_t n) const noexcept {
  auto data = reinterpret_cast<const char *>(this->data());
  return *reinterpret_cast<const T *>(data + n * this->stride());
}

template <typename T>
const T &Vec<T>::at(size_t n) const {
  if (n >= this->size()) {
    panic<std::out_of_range>("rust::Vec index out of range");
  }
  return (*this)[n];
}

template <typename T>
const T &Vec<T>::front() const {
  return (*this)[0];
}

template <typename T>
const T &Vec<T>::back() const {
  return (*this)[this->size() - 1];
}

template <typename T>
void Vec<T>::reserve(size_t new_cap) {
  this->reserve_total(new_cap);
}

template <typename T>
void Vec<T>::push_back(const T &value) {
  this->emplace_back(value);
}

template <typename T>
void Vec<T>::push_back(T &&value) {
  this->emplace_back(std::move(value));
}

template <typename T>
template <typename... Args>
void Vec<T>::emplace_back(Args &&... args) {
  auto size = this->size();
  this->reserve_total(size + 1);
  ::new (reinterpret_cast<T *>(reinterpret_cast<char *>(this->data()) +
                               size * this->stride()))
      T(std::forward<Args>(args)...);
  this->set_len(size + 1);
}

template <typename T>
const T &Vec<T>::const_iterator::operator*() const noexcept {
  return *static_cast<const T *>(this->pos);
}

template <typename T>
const T *Vec<T>::const_iterator::operator->() const noexcept {
  return static_cast<const T *>(this->pos);
}

template <typename T>
typename Vec<T>::const_iterator &Vec<T>::const_iterator::operator++() noexcept {
  this->pos = static_cast<const uint8_t *>(this->pos) + this->stride;
  return *this;
}

template <typename T>
typename Vec<T>::const_iterator
Vec<T>::const_iterator::operator++(int) noexcept {
  auto ret = const_iterator(*this);
  this->pos = static_cast<const uint8_t *>(this->pos) + this->stride;
  return ret;
}

template <typename T>
bool Vec<T>::const_iterator::operator==(
    const const_iterator &other) const noexcept {
  return this->pos == other.pos;
}

template <typename T>
bool Vec<T>::const_iterator::operator!=(
    const const_iterator &other) const noexcept {
  return this->pos != other.pos;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::begin() const noexcept {
  const_iterator it;
  it.pos = this->data();
  it.stride = this->stride();
  return it;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::end() const noexcept {
  const_iterator it = this->begin();
  it.pos = static_cast<const uint8_t *>(it.pos) + it.stride * this->size();
  return it;
}

template <typename T>
Vec<T>::Vec(unsafe_bitcopy_t, const Vec &bits) noexcept : repr(bits.repr) {}
#endif // CXXBRIDGE05_RUST_VEC

#ifndef CXXBRIDGE05_RUST_ISIZE
#define CXXBRIDGE05_RUST_ISIZE
#if defined(_WIN32)
using isize = SSIZE_T;
#else
using isize = ssize_t;
#endif
#endif // CXXBRIDGE05_RUST_ISIZE
} // namespace cxxbridge05
} // namespace rust

namespace newsboat {
  namespace utils {
    struct FilterUrlParts;
  }
}

namespace newsboat {
namespace utils {
#ifndef CXXBRIDGE05_STRUCT_newsboat$utils$FilterUrlParts
#define CXXBRIDGE05_STRUCT_newsboat$utils$FilterUrlParts
struct FilterUrlParts final {
  ::rust::String script_name;
  ::rust::String url;
};
#endif // CXXBRIDGE05_STRUCT_newsboat$utils$FilterUrlParts

uint32_t get_random_value(uint32_t max) noexcept;

bool is_special_url(::rust::Str url) noexcept;

bool is_http_url(::rust::Str url) noexcept;

bool is_query_url(::rust::Str url) noexcept;

bool is_filter_url(::rust::Str url) noexcept;

bool is_exec_url(::rust::Str url) noexcept;

bool is_valid_color(::rust::Str color) noexcept;

bool is_valid_attribute(::rust::Str attribute) noexcept;

size_t gentabs(::rust::Str string) noexcept;

void run_command(::rust::Str cmd, ::rust::Str param) noexcept;

::rust::isize strnaturalcmp(::rust::Str a, ::rust::Str b) noexcept;

size_t strwidth(::rust::Str rs_str) noexcept;

size_t strwidth_stfl(::rust::Str rs_str) noexcept;

::newsboat::utils::FilterUrlParts extract_filter(::rust::Str line) noexcept;

uint32_t newsboat_major_version() noexcept;

uint64_t get_auth_method(::rust::Str method) noexcept;

namespace bridged {
uint32_t to_u(::rust::String input, uint32_t default_value) noexcept;

bool run_interactively(::rust::Str command, ::rust::Str caller, uint8_t &exit_code) noexcept;

bool run_non_interactively(::rust::Str command, ::rust::Str caller, uint8_t &exit_code) noexcept;

bool read_text_file(::rust::String filename, ::rust::Vec<::rust::String> &contents, uint64_t &error_line_number, ::rust::String &error_reason) noexcept;

::rust::String replace_all(::rust::String input, ::rust::Str from, ::rust::Str to) noexcept;

::rust::String consolidate_whitespace(::rust::String input) noexcept;

::rust::String absolute_url(::rust::Str base_url, ::rust::Str link) noexcept;

::rust::String censor_url(::rust::Str url) noexcept;

::rust::String quote_for_stfl(::rust::Str string) noexcept;

::rust::String trim(::rust::String rs_str) noexcept;

::rust::String trim_end(::rust::String rs_str) noexcept;

::rust::String quote(::rust::String input) noexcept;

::rust::String quote_if_necessary(::rust::String input) noexcept;

::rust::String make_title(::rust::String rs_str) noexcept;

::rust::String get_default_browser() noexcept;

::rust::String substr_with_width(::rust::Str string, size_t max_width) noexcept;

::rust::String substr_with_width_stfl(::rust::Str string, size_t max_width) noexcept;

::rust::String get_command_output(::rust::Str cmd) noexcept;

::rust::String get_basename(::rust::Str input) noexcept;

::rust::String program_version() noexcept;

::rust::Str strip_comments(::rust::Str line) noexcept;

bool is_valid_podcast_type(::rust::Str mimetype) noexcept;

::rust::String resolve_tilde(::rust::Str path) noexcept;

::rust::String resolve_relative(::rust::Str reference, ::rust::Str path) noexcept;

::rust::String getcwd() noexcept;

::rust::isize mkdir_parents(::rust::Str path, uint32_t mode) noexcept;

::rust::String unescape_url(::rust::String url, bool &success) noexcept;

void remove_soft_hyphens(::rust::String &text) noexcept;

bool podcast_mime_to_link_type(::rust::Str mime_type, int64_t &result) noexcept;

::rust::String run_program(const ::rust::Vec<::rust::String> &argv, ::rust::Str input) noexcept;

::rust::String translit(::rust::Str tocode, ::rust::Str fromcode) noexcept;

::rust::Vec<uint8_t> utf8_to_locale(::rust::Str text) noexcept;

::rust::String locale_to_utf8(::rust::Slice<uint8_t> text) noexcept;
} // namespace bridged
} // namespace utils
} // namespace newsboat
