#include "stdexcept"
#include "stddef.h"
#include <array>
#include <cstddef>
#include <cstdint>
#include <new>
#include <string>
#include <type_traits>
#include <utility>
#if defined(_WIN32)
#include <basetsd.h>
#endif

namespace rust {
inline namespace cxxbridge05 {
// #include "rust/cxx.h"

#ifndef CXXBRIDGE05_PANIC
#define CXXBRIDGE05_PANIC
template <typename Exception>
void panic [[noreturn]] (const char *msg);
#endif // CXXBRIDGE05_PANIC

struct unsafe_bitcopy_t;

namespace {
template <typename T>
class impl;
} // namespace

#ifndef CXXBRIDGE05_RUST_STRING
#define CXXBRIDGE05_RUST_STRING
class String final {
public:
  String() noexcept;
  String(const String &) noexcept;
  String(String &&) noexcept;
  ~String() noexcept;

  String(const std::string &);
  String(const char *);
  String(const char *, size_t);

  String &operator=(const String &) noexcept;
  String &operator=(String &&) noexcept;

  explicit operator std::string() const;

  const char *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  String(unsafe_bitcopy_t, const String &) noexcept;

private:
  std::array<uintptr_t, 3> repr;
};
#endif // CXXBRIDGE05_RUST_STRING

#ifndef CXXBRIDGE05_RUST_STR
#define CXXBRIDGE05_RUST_STR
class Str final {
public:
  Str() noexcept;
  Str(const std::string &);
  Str(const char *);
  Str(const char *, size_t);
  Str(std::string &&) = delete;

  Str &operator=(const Str &) noexcept = default;

  explicit operator std::string() const;

  const char *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  Str(const Str &) noexcept = default;
  ~Str() noexcept = default;

private:
  friend impl<Str>;
  const char *ptr;
  size_t len;
};

inline const char *Str::data() const noexcept { return this->ptr; }

inline size_t Str::size() const noexcept { return this->len; }

inline size_t Str::length() const noexcept { return this->len; }
#endif // CXXBRIDGE05_RUST_STR

#ifndef CXXBRIDGE05_RUST_SLICE
#define CXXBRIDGE05_RUST_SLICE
template <typename T>
class Slice final {
public:
  Slice() noexcept;
  Slice(const T *, size_t count) noexcept;

  Slice &operator=(const Slice<T> &) noexcept = default;

  const T *data() const noexcept;
  size_t size() const noexcept;
  size_t length() const noexcept;

  Slice(const Slice<T> &) noexcept = default;
  ~Slice() noexcept = default;

private:
  friend impl<Slice>;
  const T *ptr;
  size_t len;
};

template <typename T>
Slice<T>::Slice() noexcept : ptr(reinterpret_cast<const T *>(this)), len(0) {}

template <typename T>
Slice<T>::Slice(const T *s, size_t count) noexcept : ptr(s), len(count) {}

template <typename T>
const T *Slice<T>::data() const noexcept {
  return this->ptr;
}

template <typename T>
size_t Slice<T>::size() const noexcept {
  return this->len;
}

template <typename T>
size_t Slice<T>::length() const noexcept {
  return this->len;
}
#endif // CXXBRIDGE05_RUST_SLICE

#ifndef CXXBRIDGE05_RUST_BITCOPY
#define CXXBRIDGE05_RUST_BITCOPY
struct unsafe_bitcopy_t final {
  explicit unsafe_bitcopy_t() = default;
};

constexpr unsafe_bitcopy_t unsafe_bitcopy{};
#endif // CXXBRIDGE05_RUST_BITCOPY

#ifndef CXXBRIDGE05_RUST_VEC
#define CXXBRIDGE05_RUST_VEC
template <typename T>
class Vec final {
public:
  using value_type = T;

  Vec() noexcept;
  Vec(Vec &&) noexcept;
  ~Vec() noexcept;

  Vec &operator=(Vec &&) noexcept;

  size_t size() const noexcept;
  bool empty() const noexcept;
  const T *data() const noexcept;
  T *data() noexcept;

  const T &operator[](size_t n) const noexcept;
  const T &at(size_t n) const;

  const T &front() const;
  const T &back() const;

  void reserve(size_t new_cap);
  void push_back(const T &value);
  void push_back(T &&value);
  template <typename... Args>
  void emplace_back(Args &&... args);

  class const_iterator final {
  public:
    using difference_type = ptrdiff_t;
    using value_type = typename std::add_const<T>::type;
    using pointer =
        typename std::add_pointer<typename std::add_const<T>::type>::type;
    using reference = typename std::add_lvalue_reference<
        typename std::add_const<T>::type>::type;
    using iterator_category = std::forward_iterator_tag;

    const T &operator*() const noexcept;
    const T *operator->() const noexcept;
    const_iterator &operator++() noexcept;
    const_iterator operator++(int) noexcept;
    bool operator==(const const_iterator &) const noexcept;
    bool operator!=(const const_iterator &) const noexcept;

  private:
    friend class Vec;
    const void *pos;
    size_t stride;
  };

  const_iterator begin() const noexcept;
  const_iterator end() const noexcept;

  Vec(unsafe_bitcopy_t, const Vec &) noexcept;

private:
  static size_t stride() noexcept;
  void reserve_total(size_t cap) noexcept;
  void set_len(size_t len) noexcept;
  void drop() noexcept;

  std::array<uintptr_t, 3> repr;
};

template <typename T>
Vec<T>::Vec(Vec &&other) noexcept {
  this->repr = other.repr;
  new (&other) Vec();
}

template <typename T>
Vec<T>::~Vec() noexcept {
  this->drop();
}

template <typename T>
Vec<T> &Vec<T>::operator=(Vec &&other) noexcept {
  if (this != &other) {
    this->drop();
    this->repr = other.repr;
    new (&other) Vec();
  }
  return *this;
}

template <typename T>
bool Vec<T>::empty() const noexcept {
  return size() == 0;
}

template <typename T>
T *Vec<T>::data() noexcept {
  return const_cast<T *>(const_cast<const Vec<T> *>(this)->data());
}

template <typename T>
const T &Vec<T>::operator[](size_t n) const noexcept {
  auto data = reinterpret_cast<const char *>(this->data());
  return *reinterpret_cast<const T *>(data + n * this->stride());
}

template <typename T>
const T &Vec<T>::at(size_t n) const {
  if (n >= this->size()) {
    panic<std::out_of_range>("rust::Vec index out of range");
  }
  return (*this)[n];
}

template <typename T>
const T &Vec<T>::front() const {
  return (*this)[0];
}

template <typename T>
const T &Vec<T>::back() const {
  return (*this)[this->size() - 1];
}

template <typename T>
void Vec<T>::reserve(size_t new_cap) {
  this->reserve_total(new_cap);
}

template <typename T>
void Vec<T>::push_back(const T &value) {
  this->emplace_back(value);
}

template <typename T>
void Vec<T>::push_back(T &&value) {
  this->emplace_back(std::move(value));
}

template <typename T>
template <typename... Args>
void Vec<T>::emplace_back(Args &&... args) {
  auto size = this->size();
  this->reserve_total(size + 1);
  ::new (reinterpret_cast<T *>(reinterpret_cast<char *>(this->data()) +
                               size * this->stride()))
      T(std::forward<Args>(args)...);
  this->set_len(size + 1);
}

template <typename T>
const T &Vec<T>::const_iterator::operator*() const noexcept {
  return *static_cast<const T *>(this->pos);
}

template <typename T>
const T *Vec<T>::const_iterator::operator->() const noexcept {
  return static_cast<const T *>(this->pos);
}

template <typename T>
typename Vec<T>::const_iterator &Vec<T>::const_iterator::operator++() noexcept {
  this->pos = static_cast<const uint8_t *>(this->pos) + this->stride;
  return *this;
}

template <typename T>
typename Vec<T>::const_iterator
Vec<T>::const_iterator::operator++(int) noexcept {
  auto ret = const_iterator(*this);
  this->pos = static_cast<const uint8_t *>(this->pos) + this->stride;
  return ret;
}

template <typename T>
bool Vec<T>::const_iterator::operator==(
    const const_iterator &other) const noexcept {
  return this->pos == other.pos;
}

template <typename T>
bool Vec<T>::const_iterator::operator!=(
    const const_iterator &other) const noexcept {
  return this->pos != other.pos;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::begin() const noexcept {
  const_iterator it;
  it.pos = this->data();
  it.stride = this->stride();
  return it;
}

template <typename T>
typename Vec<T>::const_iterator Vec<T>::end() const noexcept {
  const_iterator it = this->begin();
  it.pos = static_cast<const uint8_t *>(it.pos) + it.stride * this->size();
  return it;
}

template <typename T>
Vec<T>::Vec(unsafe_bitcopy_t, const Vec &bits) noexcept : repr(bits.repr) {}
#endif // CXXBRIDGE05_RUST_VEC

#ifndef CXXBRIDGE05_RUST_ISIZE
#define CXXBRIDGE05_RUST_ISIZE
#if defined(_WIN32)
using isize = SSIZE_T;
#else
using isize = ssize_t;
#endif
#endif // CXXBRIDGE05_RUST_ISIZE

template <typename T>
union MaybeUninit {
  T value;
  MaybeUninit() {}
  ~MaybeUninit() {}
};

namespace {
namespace repr {
struct PtrLen final {
  const void *ptr;
  size_t len;
};
} // namespace repr

template <>
class impl<Str> final {
public:
  static Str new_unchecked(repr::PtrLen repr) noexcept {
    Str str;
    str.ptr = static_cast<const char *>(repr.ptr);
    str.len = repr.len;
    return str;
  }
  static repr::PtrLen repr(Str str) noexcept {
    return repr::PtrLen{str.ptr, str.len};
  }
};

template <typename T>
class impl<Slice<T>> final {
public:
  static repr::PtrLen repr(Slice<T> slice) noexcept {
    return repr::PtrLen{slice.ptr, slice.len};
  }
};
} // namespace
} // namespace cxxbridge05
} // namespace rust

namespace newsboat {
  namespace utils {
    struct FilterUrlParts;
  }
}

namespace newsboat {
namespace utils {
#ifndef CXXBRIDGE05_STRUCT_newsboat$utils$FilterUrlParts
#define CXXBRIDGE05_STRUCT_newsboat$utils$FilterUrlParts
struct FilterUrlParts final {
  ::rust::String script_name;
  ::rust::String url;
};
#endif // CXXBRIDGE05_STRUCT_newsboat$utils$FilterUrlParts

extern "C" {
uint32_t newsboat$utils$cxxbridge05$get_random_value(uint32_t max) noexcept;

bool newsboat$utils$cxxbridge05$is_special_url(::rust::repr::PtrLen url) noexcept;

bool newsboat$utils$cxxbridge05$is_http_url(::rust::repr::PtrLen url) noexcept;

bool newsboat$utils$cxxbridge05$is_query_url(::rust::repr::PtrLen url) noexcept;

bool newsboat$utils$cxxbridge05$is_filter_url(::rust::repr::PtrLen url) noexcept;

bool newsboat$utils$cxxbridge05$is_exec_url(::rust::repr::PtrLen url) noexcept;

bool newsboat$utils$cxxbridge05$is_valid_color(::rust::repr::PtrLen color) noexcept;

bool newsboat$utils$cxxbridge05$is_valid_attribute(::rust::repr::PtrLen attribute) noexcept;

size_t newsboat$utils$cxxbridge05$gentabs(::rust::repr::PtrLen string) noexcept;

void newsboat$utils$cxxbridge05$run_command(::rust::repr::PtrLen cmd, ::rust::repr::PtrLen param) noexcept;

::rust::isize newsboat$utils$cxxbridge05$strnaturalcmp(::rust::repr::PtrLen a, ::rust::repr::PtrLen b) noexcept;

size_t newsboat$utils$cxxbridge05$strwidth(::rust::repr::PtrLen rs_str) noexcept;

size_t newsboat$utils$cxxbridge05$strwidth_stfl(::rust::repr::PtrLen rs_str) noexcept;

void newsboat$utils$cxxbridge05$extract_filter(::rust::repr::PtrLen line, ::newsboat::utils::FilterUrlParts *return$) noexcept;

uint32_t newsboat$utils$cxxbridge05$newsboat_major_version() noexcept;

uint64_t newsboat$utils$cxxbridge05$get_auth_method(::rust::repr::PtrLen method) noexcept;
} // extern "C"

namespace bridged {
extern "C" {
uint32_t newsboat$utils$bridged$cxxbridge05$to_u(::rust::String *input, uint32_t default_value) noexcept;

bool newsboat$utils$bridged$cxxbridge05$run_interactively(::rust::repr::PtrLen command, ::rust::repr::PtrLen caller, uint8_t &exit_code) noexcept;

bool newsboat$utils$bridged$cxxbridge05$run_non_interactively(::rust::repr::PtrLen command, ::rust::repr::PtrLen caller, uint8_t &exit_code) noexcept;

bool newsboat$utils$bridged$cxxbridge05$read_text_file(::rust::String *filename, ::rust::Vec<::rust::String> &contents, uint64_t &error_line_number, ::rust::String &error_reason) noexcept;

void newsboat$utils$bridged$cxxbridge05$replace_all(::rust::String *input, ::rust::repr::PtrLen from, ::rust::repr::PtrLen to, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$consolidate_whitespace(::rust::String *input, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$absolute_url(::rust::repr::PtrLen base_url, ::rust::repr::PtrLen link, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$censor_url(::rust::repr::PtrLen url, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$quote_for_stfl(::rust::repr::PtrLen string, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$trim(::rust::String *rs_str, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$trim_end(::rust::String *rs_str, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$quote(::rust::String *input, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$quote_if_necessary(::rust::String *input, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$make_title(::rust::String *rs_str, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$get_default_browser(::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$substr_with_width(::rust::repr::PtrLen string, size_t max_width, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$substr_with_width_stfl(::rust::repr::PtrLen string, size_t max_width, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$get_command_output(::rust::repr::PtrLen cmd, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$get_basename(::rust::repr::PtrLen input, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$program_version(::rust::String *return$) noexcept;

::rust::repr::PtrLen newsboat$utils$bridged$cxxbridge05$strip_comments(::rust::repr::PtrLen line) noexcept;

bool newsboat$utils$bridged$cxxbridge05$is_valid_podcast_type(::rust::repr::PtrLen mimetype) noexcept;

void newsboat$utils$bridged$cxxbridge05$resolve_tilde(::rust::repr::PtrLen path, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$resolve_relative(::rust::repr::PtrLen reference, ::rust::repr::PtrLen path, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$getcwd(::rust::String *return$) noexcept;

::rust::isize newsboat$utils$bridged$cxxbridge05$mkdir_parents(::rust::repr::PtrLen path, uint32_t mode) noexcept;

void newsboat$utils$bridged$cxxbridge05$unescape_url(::rust::String *url, bool &success, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$remove_soft_hyphens(::rust::String &text) noexcept;

bool newsboat$utils$bridged$cxxbridge05$podcast_mime_to_link_type(::rust::repr::PtrLen mime_type, int64_t &result) noexcept;

void newsboat$utils$bridged$cxxbridge05$run_program(const ::rust::Vec<::rust::String> &argv, ::rust::repr::PtrLen input, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$translit(::rust::repr::PtrLen tocode, ::rust::repr::PtrLen fromcode, ::rust::String *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$utf8_to_locale(::rust::repr::PtrLen text, ::rust::Vec<uint8_t> *return$) noexcept;

void newsboat$utils$bridged$cxxbridge05$locale_to_utf8(::rust::repr::PtrLen text, ::rust::String *return$) noexcept;
} // extern "C"
} // namespace bridged

uint32_t get_random_value(uint32_t max) noexcept {
  return newsboat$utils$cxxbridge05$get_random_value(max);
}

bool is_special_url(::rust::Str url) noexcept {
  return newsboat$utils$cxxbridge05$is_special_url(::rust::impl<::rust::Str>::repr(url));
}

bool is_http_url(::rust::Str url) noexcept {
  return newsboat$utils$cxxbridge05$is_http_url(::rust::impl<::rust::Str>::repr(url));
}

bool is_query_url(::rust::Str url) noexcept {
  return newsboat$utils$cxxbridge05$is_query_url(::rust::impl<::rust::Str>::repr(url));
}

bool is_filter_url(::rust::Str url) noexcept {
  return newsboat$utils$cxxbridge05$is_filter_url(::rust::impl<::rust::Str>::repr(url));
}

bool is_exec_url(::rust::Str url) noexcept {
  return newsboat$utils$cxxbridge05$is_exec_url(::rust::impl<::rust::Str>::repr(url));
}

bool is_valid_color(::rust::Str color) noexcept {
  return newsboat$utils$cxxbridge05$is_valid_color(::rust::impl<::rust::Str>::repr(color));
}

bool is_valid_attribute(::rust::Str attribute) noexcept {
  return newsboat$utils$cxxbridge05$is_valid_attribute(::rust::impl<::rust::Str>::repr(attribute));
}

size_t gentabs(::rust::Str string) noexcept {
  return newsboat$utils$cxxbridge05$gentabs(::rust::impl<::rust::Str>::repr(string));
}

void run_command(::rust::Str cmd, ::rust::Str param) noexcept {
  newsboat$utils$cxxbridge05$run_command(::rust::impl<::rust::Str>::repr(cmd), ::rust::impl<::rust::Str>::repr(param));
}

::rust::isize strnaturalcmp(::rust::Str a, ::rust::Str b) noexcept {
  return newsboat$utils$cxxbridge05$strnaturalcmp(::rust::impl<::rust::Str>::repr(a), ::rust::impl<::rust::Str>::repr(b));
}

size_t strwidth(::rust::Str rs_str) noexcept {
  return newsboat$utils$cxxbridge05$strwidth(::rust::impl<::rust::Str>::repr(rs_str));
}

size_t strwidth_stfl(::rust::Str rs_str) noexcept {
  return newsboat$utils$cxxbridge05$strwidth_stfl(::rust::impl<::rust::Str>::repr(rs_str));
}

::newsboat::utils::FilterUrlParts extract_filter(::rust::Str line) noexcept {
  ::rust::MaybeUninit<::newsboat::utils::FilterUrlParts> return$;
  newsboat$utils$cxxbridge05$extract_filter(::rust::impl<::rust::Str>::repr(line), &return$.value);
  return ::std::move(return$.value);
}

uint32_t newsboat_major_version() noexcept {
  return newsboat$utils$cxxbridge05$newsboat_major_version();
}

uint64_t get_auth_method(::rust::Str method) noexcept {
  return newsboat$utils$cxxbridge05$get_auth_method(::rust::impl<::rust::Str>::repr(method));
}

namespace bridged {
uint32_t to_u(::rust::String input, uint32_t default_value) noexcept {
  return newsboat$utils$bridged$cxxbridge05$to_u(&input, default_value);
}

bool run_interactively(::rust::Str command, ::rust::Str caller, uint8_t &exit_code) noexcept {
  return newsboat$utils$bridged$cxxbridge05$run_interactively(::rust::impl<::rust::Str>::repr(command), ::rust::impl<::rust::Str>::repr(caller), exit_code);
}

bool run_non_interactively(::rust::Str command, ::rust::Str caller, uint8_t &exit_code) noexcept {
  return newsboat$utils$bridged$cxxbridge05$run_non_interactively(::rust::impl<::rust::Str>::repr(command), ::rust::impl<::rust::Str>::repr(caller), exit_code);
}

bool read_text_file(::rust::String filename, ::rust::Vec<::rust::String> &contents, uint64_t &error_line_number, ::rust::String &error_reason) noexcept {
  return newsboat$utils$bridged$cxxbridge05$read_text_file(&filename, contents, error_line_number, error_reason);
}

::rust::String replace_all(::rust::String input, ::rust::Str from, ::rust::Str to) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$replace_all(&input, ::rust::impl<::rust::Str>::repr(from), ::rust::impl<::rust::Str>::repr(to), &return$.value);
  return ::std::move(return$.value);
}

::rust::String consolidate_whitespace(::rust::String input) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$consolidate_whitespace(&input, &return$.value);
  return ::std::move(return$.value);
}

::rust::String absolute_url(::rust::Str base_url, ::rust::Str link) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$absolute_url(::rust::impl<::rust::Str>::repr(base_url), ::rust::impl<::rust::Str>::repr(link), &return$.value);
  return ::std::move(return$.value);
}

::rust::String censor_url(::rust::Str url) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$censor_url(::rust::impl<::rust::Str>::repr(url), &return$.value);
  return ::std::move(return$.value);
}

::rust::String quote_for_stfl(::rust::Str string) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$quote_for_stfl(::rust::impl<::rust::Str>::repr(string), &return$.value);
  return ::std::move(return$.value);
}

::rust::String trim(::rust::String rs_str) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$trim(&rs_str, &return$.value);
  return ::std::move(return$.value);
}

::rust::String trim_end(::rust::String rs_str) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$trim_end(&rs_str, &return$.value);
  return ::std::move(return$.value);
}

::rust::String quote(::rust::String input) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$quote(&input, &return$.value);
  return ::std::move(return$.value);
}

::rust::String quote_if_necessary(::rust::String input) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$quote_if_necessary(&input, &return$.value);
  return ::std::move(return$.value);
}

::rust::String make_title(::rust::String rs_str) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$make_title(&rs_str, &return$.value);
  return ::std::move(return$.value);
}

::rust::String get_default_browser() noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$get_default_browser(&return$.value);
  return ::std::move(return$.value);
}

::rust::String substr_with_width(::rust::Str string, size_t max_width) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$substr_with_width(::rust::impl<::rust::Str>::repr(string), max_width, &return$.value);
  return ::std::move(return$.value);
}

::rust::String substr_with_width_stfl(::rust::Str string, size_t max_width) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$substr_with_width_stfl(::rust::impl<::rust::Str>::repr(string), max_width, &return$.value);
  return ::std::move(return$.value);
}

::rust::String get_command_output(::rust::Str cmd) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$get_command_output(::rust::impl<::rust::Str>::repr(cmd), &return$.value);
  return ::std::move(return$.value);
}

::rust::String get_basename(::rust::Str input) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$get_basename(::rust::impl<::rust::Str>::repr(input), &return$.value);
  return ::std::move(return$.value);
}

::rust::String program_version() noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$program_version(&return$.value);
  return ::std::move(return$.value);
}

::rust::Str strip_comments(::rust::Str line) noexcept {
  return ::rust::impl<::rust::Str>::new_unchecked(newsboat$utils$bridged$cxxbridge05$strip_comments(::rust::impl<::rust::Str>::repr(line)));
}

bool is_valid_podcast_type(::rust::Str mimetype) noexcept {
  return newsboat$utils$bridged$cxxbridge05$is_valid_podcast_type(::rust::impl<::rust::Str>::repr(mimetype));
}

::rust::String resolve_tilde(::rust::Str path) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$resolve_tilde(::rust::impl<::rust::Str>::repr(path), &return$.value);
  return ::std::move(return$.value);
}

::rust::String resolve_relative(::rust::Str reference, ::rust::Str path) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$resolve_relative(::rust::impl<::rust::Str>::repr(reference), ::rust::impl<::rust::Str>::repr(path), &return$.value);
  return ::std::move(return$.value);
}

::rust::String getcwd() noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$getcwd(&return$.value);
  return ::std::move(return$.value);
}

::rust::isize mkdir_parents(::rust::Str path, uint32_t mode) noexcept {
  return newsboat$utils$bridged$cxxbridge05$mkdir_parents(::rust::impl<::rust::Str>::repr(path), mode);
}

::rust::String unescape_url(::rust::String url, bool &success) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$unescape_url(&url, success, &return$.value);
  return ::std::move(return$.value);
}

void remove_soft_hyphens(::rust::String &text) noexcept {
  newsboat$utils$bridged$cxxbridge05$remove_soft_hyphens(text);
}

bool podcast_mime_to_link_type(::rust::Str mime_type, int64_t &result) noexcept {
  return newsboat$utils$bridged$cxxbridge05$podcast_mime_to_link_type(::rust::impl<::rust::Str>::repr(mime_type), result);
}

::rust::String run_program(const ::rust::Vec<::rust::String> &argv, ::rust::Str input) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$run_program(argv, ::rust::impl<::rust::Str>::repr(input), &return$.value);
  return ::std::move(return$.value);
}

::rust::String translit(::rust::Str tocode, ::rust::Str fromcode) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$translit(::rust::impl<::rust::Str>::repr(tocode), ::rust::impl<::rust::Str>::repr(fromcode), &return$.value);
  return ::std::move(return$.value);
}

::rust::Vec<uint8_t> utf8_to_locale(::rust::Str text) noexcept {
  ::rust::MaybeUninit<::rust::Vec<uint8_t>> return$;
  newsboat$utils$bridged$cxxbridge05$utf8_to_locale(::rust::impl<::rust::Str>::repr(text), &return$.value);
  return ::std::move(return$.value);
}

::rust::String locale_to_utf8(::rust::Slice<uint8_t> text) noexcept {
  ::rust::MaybeUninit<::rust::String> return$;
  newsboat$utils$bridged$cxxbridge05$locale_to_utf8(::rust::impl<::rust::Slice<uint8_t>>::repr(text), &return$.value);
  return ::std::move(return$.value);
}
} // namespace bridged
} // namespace utils
} // namespace newsboat
